"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var instance_validator_exports = {};
__export(instance_validator_exports, {
  InstanceValidator: () => InstanceValidator
});
module.exports = __toCommonJS(instance_validator_exports);
var import_data_types = require("./dialects/abstract/data-types");
var import_data_types_utils = require("./dialects/abstract/data-types-utils");
var import_object = require("./utils/object");
var import_sequelize_method = require("./utils/sequelize-method");
var import_belongs_to = require("./associations/belongs-to");
const _ = require("lodash");
const sequelizeError = require("./errors");
const validator = require("./utils/validator-extras").validator;
const { promisify } = require("node:util");
class InstanceValidator {
  constructor(modelInstance, options) {
    options = {
      // assign defined and default options
      hooks: true,
      ...options
    };
    if (options.fields && !options.skip) {
      options.skip = _.difference(Array.from(modelInstance.constructor.modelDefinition.attributes.keys()), options.fields);
    } else {
      options.skip ?? (options.skip = []);
    }
    this.options = options;
    this.modelInstance = modelInstance;
    this.validator = validator;
    this.errors = [];
    this.inProgress = false;
  }
  /**
   * The main entry point for the Validation module, invoke to start the dance.
   *
   * @returns {Promise}
   * @private
   */
  async _validate() {
    if (this.inProgress) {
      throw new Error("Validations already in progress.");
    }
    this.inProgress = true;
    await Promise.all([
      this._perAttributeValidators(),
      this._customValidators()
    ]);
    if (this.errors.length > 0) {
      throw new sequelizeError.ValidationError(null, this.errors);
    }
  }
  /**
   * Invoke the Validation sequence and run validation hooks if defined
   *   - Before Validation Model Hooks
   *   - Validation
   *   - On validation success: After Validation Model Hooks
   *   - On validation failure: Validation Failed Model Hooks
   *
   * @returns {Promise}
   * @private
   */
  async validate() {
    return await (this.options.hooks ? this._validateAndRunHooks() : this._validate());
  }
  /**
   * Invoke the Validation sequence and run hooks
   *   - Before Validation Model Hooks
   *   - Validation
   *   - On validation success: After Validation Model Hooks
   *   - On validation failure: Validation Failed Model Hooks
   *
   * @returns {Promise}
   * @private
   */
  async _validateAndRunHooks() {
    await this.modelInstance.constructor.hooks.runAsync("beforeValidate", this.modelInstance, this.options);
    try {
      await this._validate();
    } catch (error) {
      const newError = await this.modelInstance.constructor.hooks.runAsync("validationFailed", this.modelInstance, this.options, error);
      throw newError || error;
    }
    await this.modelInstance.constructor.hooks.runAsync("afterValidate", this.modelInstance, this.options);
    return this.modelInstance;
  }
  /**
   * Will run all the validators defined per attribute (built-in validators and custom validators)
   *
   * @returns {Promise<Array>}
   * @private
   */
  async _perAttributeValidators() {
    const validators = [];
    const { attributes } = this.modelInstance.constructor.modelDefinition;
    for (const attribute of attributes.values()) {
      const attrName = attribute.attributeName;
      if (this.options.skip.includes(attrName)) {
        continue;
      }
      const value = this.modelInstance.dataValues[attrName];
      if (value instanceof import_sequelize_method.SequelizeMethod) {
        continue;
      }
      if (!attribute._autoGenerated && !attribute.autoIncrement) {
        this._validateSchema(attribute, attrName, value);
      }
      if (attribute.validate) {
        validators.push(this._singleAttrValidate(value, attrName, attribute.allowNull));
      }
    }
    return await Promise.all(validators);
  }
  /**
   * Will run all the custom validators defined in the model's options.
   *
   * @returns {Promise<Array>}
   * @private
   */
  async _customValidators() {
    const validators = [];
    const validateOptions = this.modelInstance.constructor.options.validate;
    for (const validatorName of (0, import_object.getAllOwnKeys)(validateOptions)) {
      if (this.options.skip.includes(validatorName)) {
        continue;
      }
      const validator2 = validateOptions[validatorName];
      const valprom = this._invokeCustomValidator(validator2, validatorName).catch(() => {
      });
      validators.push(valprom);
    }
    return await Promise.all(validators);
  }
  /**
   * Validate a single attribute with all the defined built-in validators and custom validators.
   *
   * @private
   *
   * @param {*} value Anything.
   * @param {string} attributeName The attribute name.
   * @param {boolean} allowNull Whether or not the schema allows null values
   *
   * @returns {Promise} A promise, will always resolve, auto populates error on this.error local object.
   */
  async _singleAttrValidate(value, attributeName, allowNull) {
    if (value == null && !allowNull) {
      return;
    }
    const validators = [];
    const attribute = this.modelInstance.constructor.modelDefinition.attributes.get(attributeName);
    _.forIn(attribute.validate, (test, validatorType) => {
      if (["isUrl", "isURL", "isEmail"].includes(validatorType)) {
        if (typeof test === "object" && test !== null && test.msg) {
          test = {
            msg: test.msg
          };
        } else if (test === true) {
          test = {};
        }
      }
      if (typeof test === "function") {
        validators.push(this._invokeCustomValidator(test, validatorType, true, value, attributeName));
        return;
      }
      if (value === null || value === void 0) {
        return;
      }
      const validatorPromise = this._invokeBuiltinValidator(value, test, validatorType, attributeName);
      validatorPromise.catch(() => {
      });
      validators.push(validatorPromise);
    });
    return Promise.all(validators.map((validator2) => validator2.catch((error) => {
      const isBuiltIn = Boolean(error.validatorName);
      this._pushError(isBuiltIn, attributeName, error, value, error.validatorName, error.validatorArgs);
    })));
  }
  /**
   * Prepare and invoke a custom validator.
   *
   * @private
   *
   * @param {Function} validator The custom validator.
   * @param {string} validatorType the custom validator type (name).
   * @param {boolean} optAttrDefined Set to true if custom validator was defined from the attribute
   * @param {*} optValue value for attribute
   * @param {string} optField field for attribute
   *
   * @returns {Promise} A promise.
   */
  async _invokeCustomValidator(validator2, validatorType, optAttrDefined, optValue, optField) {
    let isAsync = false;
    const validatorArity = validator2.length;
    let asyncArity = 1;
    let errorKey = validatorType;
    let invokeArgs;
    if (optAttrDefined) {
      asyncArity = 2;
      invokeArgs = optValue;
      errorKey = optField;
    }
    if (validatorArity === asyncArity) {
      isAsync = true;
    }
    if (isAsync) {
      try {
        if (optAttrDefined) {
          return await promisify(validator2.bind(this.modelInstance, invokeArgs))();
        }
        return await promisify(validator2.bind(this.modelInstance))();
      } catch (error) {
        return this._pushError(false, errorKey, error, optValue, validatorType);
      }
    }
    try {
      return await validator2.call(this.modelInstance, invokeArgs);
    } catch (error) {
      return this._pushError(false, errorKey, error, optValue, validatorType);
    }
  }
  /**
   * Prepare and invoke a build-in validator.
   *
   * @private
   *
   * @param {*} value Anything.
   * @param {*} test The test case.
   * @param {string} validatorType One of known to Sequelize validators.
   * @param {string} field The field that is being validated
   *
   * @returns {object} An object with specific keys to invoke the validator.
   */
  async _invokeBuiltinValidator(value, test, validatorType, field) {
    const valueString = String(value);
    if (typeof validator[validatorType] !== "function") {
      throw new TypeError(`Invalid validator function: ${validatorType}`);
    }
    const validatorArgs = this._extractValidatorArgs(test, validatorType, field);
    if (!validator[validatorType](valueString, ...validatorArgs)) {
      throw Object.assign(new Error(test.msg || `Validation ${validatorType} on ${field} failed`), { validatorName: validatorType, validatorArgs });
    }
  }
  /**
   * Will extract arguments for the validator.
   *
   * @param {*} test The test case.
   * @param {string} validatorType One of known to Sequelize validators.
   * @param {string} field The field that is being validated.
   *
   * @private
   */
  _extractValidatorArgs(test, validatorType, field) {
    let validatorArgs = test.args || test;
    const isLocalizedValidator = typeof validatorArgs !== "string" && ["isAlpha", "isAlphanumeric", "isMobilePhone"].includes(validatorType);
    if (!Array.isArray(validatorArgs)) {
      if (validatorType === "isImmutable") {
        validatorArgs = [validatorArgs, field, this.modelInstance];
      } else if (isLocalizedValidator || validatorType === "isIP") {
        validatorArgs = [];
      } else {
        validatorArgs = [validatorArgs];
      }
    } else {
      validatorArgs = [...validatorArgs];
    }
    return validatorArgs;
  }
  /**
   * Will validate a single field against its schema definition (isnull).
   *
   * @param {object} attribute As defined in the Schema.
   * @param {string} attributeName The field name.
   * @param {*} value anything.
   *
   * @private
   */
  _validateSchema(attribute, attributeName, value) {
    var _a;
    if (attribute.allowNull === false && value == null) {
      const association = Object.values(this.modelInstance.constructor.associations).find((association2) => association2 instanceof import_belongs_to.BelongsTo && association2.foreignKey === attribute.fieldName);
      if (!association || !this.modelInstance.get(association.as)) {
        const modelDefinition = this.modelInstance.constructor.modelDefinition;
        const validators = (_a = modelDefinition.attributes.get(attributeName)) == null ? void 0 : _a.validate;
        const errMsg = _.get(validators, "notNull.msg", `${this.modelInstance.constructor.name}.${attributeName} cannot be null`);
        this.errors.push(new sequelizeError.ValidationErrorItem(
          errMsg,
          "notNull violation",
          // sequelizeError.ValidationErrorItem.Origins.CORE,
          attributeName,
          value,
          this.modelInstance,
          "is_null"
        ));
      }
    }
    const type = attribute.type;
    if (value != null && !(value instanceof import_sequelize_method.SequelizeMethod) && type instanceof import_data_types.AbstractDataType) {
      const error = (0, import_data_types_utils.validateDataType)(type, attributeName, this.modelInstance, value);
      if (error) {
        this.errors.push(error);
      }
    }
  }
  /**
   * Signs all errors retaining the original.
   *
   * @param {boolean}       isBuiltin   - Determines if error is from builtin validator.
   * @param {string}        errorKey    - name of invalid attribute.
   * @param {Error|string}  rawError    - The original error.
   * @param {string|number} value       - The data that triggered the error.
   * @param {string}        fnName      - Name of the validator, if any
   * @param {Array}         fnArgs      - Arguments for the validator [function], if any
   *
   * @private
   */
  _pushError(isBuiltin, errorKey, rawError, value, fnName, fnArgs) {
    const message = rawError.message || rawError || "Validation error";
    const error = new sequelizeError.ValidationErrorItem(
      message,
      "Validation error",
      // sequelizeError.ValidationErrorItem.Origins.FUNCTION,
      errorKey,
      value,
      this.modelInstance,
      fnName,
      isBuiltin ? fnName : void 0,
      isBuiltin ? fnArgs : void 0
    );
    error[InstanceValidator.RAW_KEY_NAME] = rawError;
    this.errors.push(error);
  }
}
InstanceValidator.RAW_KEY_NAME = "original";
//# sourceMappingURL=instance-validator.js.map
